function alarmResult=challenge(recordName,alarm_type)
%
% Last Modified: 18 July 2015 - JHuvanandana
% 
% alarmResult=challenge(recordName,alarm_type)
%
% Sample entry for the 2015 PhysioNet/CinC Challenge.
%
% Inputs:
%   recordName
%       String specifying the record name to process
%   alarmType
%       String specifying the alarm type. Alarm types are:
%             Asystole, Bradycardia, Tachycardia, Ventricular_Tachycardia,
%             Ventricular_Flutter_Fib
%
% 
% Outputs:
%   alarmResult
%       Integer value where 0 = false alarm and 1 is a true alarm.
%
%
% To run your entry on the entire training set in a format that is
% compatible with PhysioNet's scoring enviroment, run the script
% generateValidationSet.m
%
% Dependencies:
%
%       1) This function does not requires that you have the WFDB App
%       Toolbox installed. A matlab function 'rdmat' can read the data
%       instead of using WFDB Toolbox.
%
%       2) The CHALLENGE function requires that you have downloaded the
%       challenge data 'set-p' in a subdirectory of the current directory.
%       The subdirectory should be called '/challenge/2015/set-p/' . The
%       'set-p' dataset can be downloaded from PhysioNet at:
%           http://physionet.org/physiobank/database/challenge/2015/
%          
%         This dataset is used by the generateValidationSet.m script to
%         create the annotations on your traing set that will be used to
%         verify that your entry works properly in the PhysioNet testing
%         environment.
%
% Version 0.5
%
%
% Written by Qiao Li, November 10, 2014. Last Modified: Ikaro Silva
% February 11, 2015
%
%
%
% %Example using training data-
% alarmResult=challenge('./challenge/set-p/100','Asystole')
%

% Name of file containing answers
answers = 'answers.txt';

%Get all ECG, blood pressure and photoplethysmogram signals
[~,signal,Fs,siginfo]=rdmat(recordName);
alarmResult=1;

% % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Initialise
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

description=squeeze(struct2cell(siginfo));
description=description(4,:);

% Resample signal to 125Hz
Fs=Fs(1);
if Fs~=125
    signal=resample(signal,125,Fs);
    Fs=125;
end


% % Adding ECG signals

%***********************  Preprocessing  ********************************
% ECGIIqrs = medfilt1(signal(:,1),20);              % 2 median filters to remove Baseline  with 200ms & 600 ms width
% ECGII_MedFilt = medfilt1(ECGIIqrs,60);
% % ECGIIqrs=sortfilt1(ECGIIraw,20,50);
% % ECGIIt=sortfilt1(ECGIIqrs,60,50);
% ecg2_Preproc = signal(:,1) - ECGII_MedFilt;
% ecg2_signal = ecg2_Preproc;
temp_signal2 = signal(:,1);
if ~isempty(temp_signal2)
  ECG2qrs = medfilt1(temp_signal2,20);
  ECG2_MedFilt = medfilt1(ECG2qrs,60);
  if length(ECG2_MedFilt)==length(temp_signal2)
     ecg2_Preproc = temp_signal2 - ECG2_MedFilt;
  else
     ecg2_Preproc = [];
  end;
else
  ecg2_Preproc=[];
end
ecg2_signal = ecg2_Preproc;

% ECGVqrs = medfilt1(signal(:,2),20);
% ECGV_MedFilt = medfilt1(ECGVqrs,60);
% ecg5_Preproc = signal(:,2) - ECGV_MedFilt;
% ecg5_signal = ecg5_Preproc;
temp_signal5 = signal(:,2);
if ~isempty(temp_signal5)
  ECGVqrs = medfilt1(temp_signal5,20);
  ECGV_MedFilt = medfilt1(ECGVqrs,60);
  if length(ECGV_MedFilt)==length(temp_signal5)
     ecg5_Preproc = temp_signal5 - ECGV_MedFilt;
  else
     ecg5_Preproc = [];
  end;
else
  ecg5_Preproc=[];
end
ecg5_signal = ecg5_Preproc;

% *********************** NaN Modification *******************************
% Check for NaN values in the ECGs and replace them with zero(previous part of
% ECG (mirroring)), then apply Hilbert QRS detection (as the Hilbert
% algorithm stops when gets to NaN) 23/7/15 Nadi:
% [iNaN, RpeaksMod_ecg2, RpeaksNaN2, ECG2_NaN, ecg2_Preproc] = ecg_NaNModify(ecg2_Preproc, Fs);
% [iNaN5, RpeaksMod_ecg5, RpeaksNaN5, ECG5_NaN, ecg5_Preproc] = ecg_NaNModify(ecg5_Preproc, Fs);
% [RpeaksMod_ecg2] = ecg_NaNModify(ecg2_Preproc, Fs);
% [RpeaksMod_ecg5] = ecg_NaNModify(ecg5_Preproc, Fs);
iNaN = find(isnan(ecg2_Preproc));
if isempty(iNaN)
    Rpeaks_NaNecg2 = QRSdet_Hilbert(ecg2_Preproc',Fs);
else
    ecg2_Preproc(iNaN) = 0;
    Rpeaks_NaNecg2 = QRSdet_Hilbert(ecg2_Preproc',Fs);
end
% ECGIINaN = ecg2_Preproc;
% Removing the replaced zeroes from ECG signal and R peaks after QRS
% detected to the end of the signal
ecg2_Preproc(iNaN) = NaN;
% In record 123; 4 detected Rpeaks are over ECG length!: So any Rpeaks over ECG length removed
RpeaksOverECG = find(Rpeaks_NaNecg2>length(ecg2_Preproc));
Rpeaks_NaNecg2(RpeaksOverECG) = [];
% RpeaksNaN = Rpeaks_NaNecg2;
% Remove the Rpeaks falling in the NaN ECG segments
kk = find(isnan(ecg2_Preproc(Rpeaks_NaNecg2)));
Rpeaks_NaNecg2(kk)=[];
Rpeaks_ecg2 = Rpeaks_NaNecg2;

% Same NaN detections and replacing zeroes for ECGV for proper performance of Hilbert QRS detection
iNaN5 = find(isnan(ecg5_Preproc));
if isempty(iNaN5)
    Rpeaks_NaNecg5 = QRSdet_Hilbert(ecg5_Preproc',Fs);
else
    ecg5_Preproc(iNaN5)= 0;
    Rpeaks_NaNecg5 = QRSdet_Hilbert(ecg5_Preproc',Fs);
end
% ECGVNaN=ecg5_Preproc;
ecg5_Preproc(iNaN5)=NaN;
RpeaksOverECGV=find(Rpeaks_NaNecg5>length(ecg5_Preproc));
Rpeaks_NaNecg5(RpeaksOverECGV)=[];
% RpeaksNaN5=Rpeaks_NaNecg5;
kk5=find(isnan(ecg5_Preproc(Rpeaks_NaNecg5)));
Rpeaks_NaNecg5(kk5)=[];
Rpeaks_ecg5 = Rpeaks_NaNecg5;


%%Users can access the raw samples of the record by running the following
%command if WFDB Toolbox installed: [tm,signal]=rdsamp(recordName);
%
%%For more information please see the help in RDSAMP

%Run WABP on the record, which by default will analyze the first ABP, ART,
%or BP signal
N=[];
N0=[];
abp_ind=get_index(description,'ABP');
ann_abp=[];
features=[];
BEATQ=[];
R=[];
abp_sig = [];

if(~isempty(abp_ind))
   ann_abp=wabp(signal(:,abp_ind),0,1);
   % Analyze the signal quality index of ABP using jSQI
   if length(ann_abp)>=3 % at least 3 abp beats detected
        [features] = abpfeature(signal(:,abp_ind),ann_abp);
        abp_sig   = signal(:,abp_ind);
        [BEATQ R] = jSQI(features, ann_abp, abp_sig);
   end
end

%Run WABP on the record of 'PLETH' to analyze photoplethysmogram signal
ppg_ind=get_index(description,'PLETH');
ann_ppg=[];
if (~isempty(ppg_ind))
    y=quantile(signal(:,ppg_ind),[0.05,0.5,0.95]);
    ann_ppg=wabp(signal(:,ppg_ind),0,(y(3)-y(1))/120);
    % Analyze the signal quality index of PPG
    if ~isempty(ann_ppg)
        [psqi]=ppgSQI(signal(:,ppg_ind),ann_ppg);
    end
end

%Make decisions

% set valid data segment for decision making, 16s before the alarm
N_d=Fs*5*60; % alarm position
N0_d=N_d-Fs*16+1; % 16s before the alarm
ecg2_sig_sect = ecg2_signal(N0_d:N_d);
ecg5_sig_sect = ecg5_signal(N0_d:N_d);

abp_sig_sect = [];
if ~isempty(abp_sig)
    abp_sig_sect = abp_sig(N_d:N0_d);
end

% select the beats in the segment
n_abp_beats=intersect(find(ann_abp>=N0_d),find(ann_abp<=N_d));
n_ppg_beats=intersect(find(ann_ppg>=N0_d),find(ann_ppg<=N_d));
%Added ECG II
n_ECG2_beats=intersect(find(Rpeaks_ecg2>=N0_d),find(Rpeaks_ecg2<=N_d));
%Added ECG V
n_ECG5_beats=intersect(find(Rpeaks_ecg5>=N0_d),find(Rpeaks_ecg5<=N_d));

hr_max_abp=NaN;
max_rr_abp=NaN;
max_rr_abp=NaN;
hr_max_ppg=NaN;
max_rr_ppg=NaN;
hr_max_ECG2=NaN;
max_rr_ECG2=NaN;
% hr_max_ECG5=NaN; max_rr_ECG5=NaN;

% calculate the heart rate
if length(n_abp_beats)>=2
    hr_max_abp=60*Fs/min(diff(ann_abp(n_abp_beats)));
    max_rr_abp=max(diff(ann_abp(n_abp_beats)))/Fs;
else
    hr_max_abp=0;
    max_rr_abp=0;
end
if length(n_ppg_beats)>=2
    hr_max_ppg=60*Fs/min(diff(ann_ppg(n_ppg_beats)));
    max_rr_ppg=max(diff(ann_ppg(n_ppg_beats)))/Fs;
else
    hr_max_ppg=0;
    max_rr_ppg==0;
end
%Added ECG II
if length(n_ECG2_beats)>=2
    hr_max_ECG2=60*Fs/min(diff(Rpeaks_ecg2(n_ECG2_beats)));
    max_rr_ECG2=max(diff(Rpeaks_ecg2(n_ECG2_beats)))/Fs;
else
    hr_max_ECG2=0;
    max_rr_ECG2=0;
end
%Added ECG V
if length(n_ECG5_beats)>=2
    hr_max_ECG5=60*Fs/min(diff(Rpeaks_ecg5(n_ECG5_beats)));
    max_rr_ECG5=max(diff(Rpeaks_ecg5(n_ECG5_beats)))/Fs;
else
    hr_max_ECG5=0;
    max_rr_ECG5=0;
end

% calculate the signal quality index
if ~isempty(ann_abp)
    abpsqi=1-sum(sum(BEATQ(intersect(n_abp_beats,1:length(BEATQ)),:)))/numel(BEATQ(intersect(n_abp_beats,1:length(BEATQ)),:));
else
    abpsqi=0;
end
if ~isempty(ann_ppg)
    ppgsqi=mean(psqi(intersect(n_ppg_beats,1:length(psqi))));
else
    ppgsqi=0;
end

mx_rr_ecg2=NaN;
mx_rr_ecg5=NaN;

ecg2_beats = Rpeaks_ecg2(n_ECG2_beats);
ecg2_rr = [];
ecg2_bpm = [];
mx_rr_ecg2 = 0;
if ~isempty(ecg2_beats)
    ecg2_beats(end+1) = N_d;
    ecg2_beats        = sort(unique(ecg2_beats));
    ecg2_rr           = diff(ecg2_beats);    
    ecg2_bpm          = (60*Fs)./ecg2_rr;
    mx_rr_ecg2        = max(ecg2_rr)/Fs;
end

ecg5_beats = Rpeaks_ecg5(n_ECG5_beats);
ecg5_rr = [];
ecg5_bpm = [];
mx_rr_ecg5 = 0;
if ~isempty(ecg5_beats)
    ecg5_beats(end+1) = N_d;
    ecg5_beats        = sort(unique(ecg5_beats));
    ecg5_rr           = diff(ecg5_beats);    
    ecg5_bpm          = (60*Fs)./ecg5_rr;
    mx_rr_ecg5        = max(ecg5_rr)/Fs;
end

abp_beats = ann_abp(n_abp_beats);
abp_rr = [];
abp_bpm = [];
mx_rr_abp = 0;
if ~isempty(abp_beats)
    abp_beats(end+1) = N_d;
    abp_beats        = sort(unique(abp_beats));
    abp_rr           = diff(abp_beats);
    abp_bpm          = (60*Fs)./abp_rr;
    mx_rr_abp        = max(abp_rr)/Fs;
end

ppg_beats = ann_ppg(n_ppg_beats);
ppg_rr = [];
ppg_bpm = [];
mx_rr_ppg = 0;
if ~isempty(ppg_beats)
    ppg_beats(end+1) = N_d;
    ppg_beats        = sort(unique(ppg_beats)); 
    ppg_rr           = diff(ppg_beats);
    ppg_bpm          = (60*Fs)./ppg_rr;
    mx_rr_ppg        = max(ppg_rr)/Fs;
end

% SQI threshold
sqi_th = 0.95;

% Alarm threshold (seconds)
ASY_min_s_th = 4;
ASY_s_tol    = 0.5;

BRA_max_bpm_th = 45; %40;
BRA_min_bts    = 10; %4;

TAC_min_th  = 110; %140;
TAC_min_bts = 11; %14;

VTA_th = 70; %100;

VFB_max_bpm_th = 150;
VFB_min_bpm_th = 300;
VFB_pwr_th     = 100;
VFB_min_bts    = 10;
tolerance  = 10; % tolerance = 5 bmp

sd_ecg = std(ecg2_sig_sect);
sd_th = 0.05;

sd_ecg5 = std(ecg5_sig_sect);

% Using the original signals (without prep. and NaN mod.) for VT and VF:

ecg2_signal1 = signal(:,1);
ecg2_signal1 = medfilt1(ecg2_signal1);
Rpeak_ecg2 = QRSdet_Hilbert(ecg2_signal1',Fs);
ecg5_signal1 = signal(:,2);
ecg5_signal1 = medfilt1(ecg5_signal1);
Rpeak_ecg5 = QRSdet_Hilbert(ecg5_signal1',Fs);
ecg2_sig_sec = ecg2_signal1(N0_d:N_d);
ecg5_sig_sec = ecg5_signal1(N0_d:N_d);
n_ECG2_beat=intersect(find(Rpeak_ecg2>=N0_d),find(Rpeak_ecg2<=N_d));
n_ECG5_beat=intersect(find(Rpeak_ecg5>=N0_d),find(Rpeak_ecg5<=N_d));
mmx_rr_ecg2=NaN;
mmx_rr_ecg5=NaN;
if length(n_ECG2_beat)>=2
    hr_mmax_ECG2=60*Fs/min(diff(Rpeak_ecg2(n_ECG2_beat)));
    mmax_rr_ECG2=max(diff(Rpeak_ecg2(n_ECG2_beat)))/Fs;
else
    hr_mmax_ECG2=0;
    mmax_rr_ECG2=0;
end
if length(n_ECG5_beat)>=2
    hr_mmax_ECG5=60*Fs/min(diff(Rpeak_ecg5(n_ECG5_beat)));
    mmax_rr_ECG5=max(diff(Rpeak_ecg5(n_ECG5_beat)))/Fs;
else
    hr_mmax_ECG5=0;
    mmax_rr_ECG5=0;
end

ecg2_beat = Rpeak_ecg2(n_ECG2_beat);
ecg2_rr1 = [];
ecg2_bpm1 = [];
mmx_rr_ecg2 = 0;
if ~isempty(ecg2_beat)
    ecg2_beat(end+1) = N_d;
    ecg2_beat        = sort(unique(ecg2_beat));
    ecg2_rr1           = diff(ecg2_beat);    
    ecg2_bpm1          = (60*Fs)./ecg2_rr1;
    mmx_rr_ecg2        = max(ecg2_rr)/Fs;
end

ecg5_beat = Rpeak_ecg5(n_ECG5_beat);
ecg5_rr1 = [];
ecg5_bpm1 = [];
mmx_rr_ecg5 = 0;
if ~isempty(ecg5_beat)
    ecg5_beat(end+1) = N_d;
    ecg5_beat        = sort(unique(ecg5_beat));
    ecg5_rr1           = diff(ecg5_beat);    
    ecg5_bpm1          = (60*Fs)./ecg5_rr1;
    mmx_rr_ecg5        = max(ecg5_rr)/Fs;
end

BRA_th = 50; %40;
VFB_th = 250;

switch alarm_type
    case 'Asystole' 
        % No QRS for at least 4 seconds
        % if the signal quality is good enough and the maximum RR interval
        % is less than the Asystole threshold, set the alarm as 'F'
        if ~isempty(ecg2_beats) && mx_rr_ecg2<6 && mx_rr_ecg2>0 && sd_ecg > sd_th
            alarmResult=logical(mx_rr_ecg2>=ASY_min_s_th-ASY_s_tol);
        elseif ~isempty(ecg5_beats) && mx_rr_ecg5<6 && mx_rr_ecg5>0 && sd_ecg5 > sd_th
            alarmResult=logical(mx_rr_ecg5>=ASY_min_s_th-ASY_s_tol);
        elseif ~isempty(abp_beats)
            alarmResult=logical(mx_rr_abp>=ASY_min_s_th-ASY_s_tol);
        elseif ~isempty(ppg_beats)
            alarmResult=logical(mx_rr_ppg>=ASY_min_s_th-ASY_s_tol);
        end
    case 'Bradycardia'
        % Heart rate lower than 40 bpm for 5 consecutive beats
        % calculate low heart rate of 5 consecutive beats for Bradycardia
        low_hr_abp=NaN;
        low_hr_ppg=NaN;
        if length(n_abp_beats>=5)
            for i=1:length(n_abp_beats)-4
                low_hr_abp(i)=60*Fs/((ann_abp(n_abp_beats(i+4))-ann_abp(n_abp_beats(i)))/4);
            end
        end
        low_hr_abp=min(low_hr_abp);
        if length(n_ppg_beats>=5)
            for i=1:length(n_ppg_beats)-4
                low_hr_ppg(i)=60*Fs/((ann_ppg(n_ppg_beats(i+4))-ann_ppg(n_ppg_beats(i)))/4);
            end
        end
        low_hr_ppg=min(low_hr_ppg);
        
        ASY_th = 4;sqi_th = 0.9;tolerance = 5;
        if (abpsqi>=sqi_th && low_hr_abp-tolerance>BRA_th) || (ppgsqi>=sqi_th && low_hr_ppg-tolerance>BRA_th)
            alarmResult=0;
        end
%         all_beats=[];
%         if length(ecg2_beats)>=5 && max_rr_ECG2<6 && max_rr_ECG2>0
%             all_beats=ecg2_beats(1:end-1);
%         elseif length(ecg5_beats)>=5 && max_rr_ECG5<6 && max_rr_ECG5>0
%             all_beats=ecg5_beats(1:end-1);
%         elseif length(abp_beats)>=5 && abpsqi>=sqi_th
%             all_beats=abp_beats(1:end-1);
%         elseif length(ppg_beats)>=5 && ppgsqi>=sqi_th
%             all_beats=ppg_beats(1:end-1);
%         end
%         i=1;
%         interval = 6.66*Fs;
%         brady_found = 0;
%         if ~isempty(all_beats)
%             for i=1:length(all_beats)-5
%                 start = all_beats(i);
%                 beats = all_beats(all_beats>=start & all_beats<start+interval);
%                 if length(beats)<=5
%                     brady_found=1;
%                 end
%             end
%         end
%         alarmResult=logical(brady_found==1);
%         %%%%Excerpt Ends here %%%%%

%         bpm_vec=[];
%         if length(ecg2_beats)>=5 && max_rr_ECG2<6
%             bpm_vec=ecg2_bpm;
%         elseif length(ecg5_beats)>=5 && max_rr_ECG5<6
%             bpm_vec=ecg5_bpm;
%         elseif length(abp_beats)>=5 && abpsqi>=sqi_th
%             bpm_vec=abp_bpm;
%         elseif length(ppg_beats)>=5 && ppgsqi>=sqi_th
%             bpm_vec=ppg_bpm;
%         end
%         i=1;
%         brady_count=0;
%         while i<=length(bpm_vec) && brady_count<BRA_min_bts
%             if bpm_vec(i)<=BRA_max_bpm_th+tolerance
%                 brady_count=brady_count+1;
%             else
%                 brady_count=0;
%             end
%             i = i+1;
%         end
%         alarmResult=logical(brady_count==BRA_min_bts);
    case 'Tachycardia'
        % Heart rate higher than 140 bpm for 17 consecutive beats
        bpm_vec = [];
        if max_rr_ECG2>0 && max_rr_ECG2<6 && length(ecg2_beats)>=TAC_min_bts
            bpm_vec = ecg2_bpm;
        elseif  length(ecg5_beats)>=TAC_min_bts  && max_rr_ECG5<6 && max_rr_ECG5>0
            bpm_vec = ecg5_bpm;
        elseif abpsqi>=sqi_th && length(abp_beats)>=TAC_min_bts
            bpm_vec = abp_bpm;
        elseif ppgsqi>=sqi_th && length(ppg_beats)>=TAC_min_bts
            bpm_vec = ppg_bpm;
        end
        over_thr = bpm_vec(bpm_vec>=TAC_min_th-tolerance);
        if length(bpm_vec)>=30 || length(over_thr)>=TAC_min_bts
            alarmResult=1;
        else
            alarmResult=0;
        end

%%%START%%%
    case 'Ventricular_Tachycardia'
        % 5 or more ventricular beats with heart rate higher than 100 bpm
%         if mx_rr_ecg2<4
%             alarmResult=vtach_detect(ecg2_sig_sect, ecg2_beats, N0_d,Fs);
%         else
%             alarmResult=vtach_detect(ecg5_sig_sect, ecg5_beats, N0_d,Fs);
%         end
        % Potential take this into a function
        
        
        if ~isempty(ecg2_beat)
            i_sel_beat = ecg2_beat;
            i_sel_sigsec = ecg2_sig_sec;
        elseif ~isempty(ecg5_beat);
            i_sel_beat = ecg5_beat;
            i_sel_sigsec = ecg5_sig_sec;
        else
            i_sel_beat = [];
        end
        
        if ~isempty(i_sel_beat)
            sd                = std(i_sel_sigsec);
            temp_beats        = i_sel_beat-N0_d+1;
            temp_dif          = diff(temp_beats);
            compare           = temp_dif(1);
            re_search         = [];
            for i=2:length(temp_dif)
                this = temp_dif(i);
                if this >= 1.5*compare
                    re_search(end+1) = i;
                end
                compare = this;
            end
            
            to_add = [];
            for i=re_search
                i_start       = temp_beats(i);
                i_end         = temp_beats(i+1);
                rs_snip       = i_sel_sigsec(i_start:i_end);
                rs_sd         = std(rs_snip);
                rs_mean       = mean(rs_snip);
                t             = maxdet(rs_snip, rs_sd);
                t             = t(rs_snip(t)>=rs_mean+sd);
                t             = t+i_start+N0_d-1;
                to_add        = [to_add, t];
            end
            
            temp       = [i_sel_beat(1:end-1)+1, to_add];
            temp       = sort(unique(temp));
            i_sel_beat = temp-N0_d;
            
            % Set template based on first waveform
            mx_a     = i_sel_beat(1);
            mx_b     = i_sel_beat(2);
            snip     = i_sel_sigsec(mx_a:mx_b-1);
            mnsn     = mean(snip);
            % Remove Mean
            template = snip-mnsn;
            
            filtvec = [];
            for i=2:length(i_sel_beat)-1
                snip = i_sel_sigsec(i_sel_beat(i):i_sel_beat(i+1)-1);
                mnsn = mean(snip);
                snip = snip-mnsn;
                insig = resample(snip,length(template),length(snip));
                cc    = std(abs(template-insig));
                if cc >=0.6*sd
                    filtvec(end+1) = cc;
                end
            end
            if length(filtvec)>=4
                alarmResult = 1;
            else
                alarmResult = 0;
            end
        else
            alarmResult = 0;
        end
%%%END%%%

        
    case 'Ventricular_Flutter_Fib'
%         % Fibrillatory, flutter, or oscillatory waveform for at least 4 seconds
%         if mmax_rr_ECG2<6 && length(n_ECG2_beat)>=VFB_min_bts
%             bpm_vec = ecg2_bpm1(1:end-1); % exclude the appended final beat
%         elseif length(n_abp_beats)>=VFB_min_bts
%             bpm_vec = abp_bpm(1:end-1); % exclude the appended final beat
%         elseif length(n_ppg_beats)>=VFB_min_bts
%             bpm_vec = ppg_bpm(1:end-1); % exclude the appended final beat
%         else
%             bpm_vec = [];
%         end
%         
%         
%         % Trying to oscillatory waveform in 10-20 Hz Freq range
%         [spec,sf] = specgram(ecg2_sig_sec,Fs);
%         power     = powercalc(spec,10,20,sf);
%         
%         if max(bpm_vec)<=VFB_max_bpm_th
%             alarmResult=0;
%         elseif max(bpm_vec)>=VFB_min_bpm_th-tolerance
%             alarmResult=1;
%         elseif power<VFB_pwr_th
%             alarmResult=0;
%         end

if (abpsqi>=sqi_th && hr_max_abp+tolerance<VFB_th) || (ppgsqi>=sqi_th && hr_max_ppg+tolerance<VFB_th)
    alarmResult=0;
end

% %%% START OF EXCERPT %%%
%         if  mmax_rr_ECG2>0 && mmax_rr_ECG2<6 && length(n_ECG2_beat)>=VFB_min_bts
%             bpm_vec = ecg2_bpm1(1:end-1); % exclude the appended final beat
%          elseif  mmax_rr_ECG5>0 && mmax_rr_ECG5<6 && length(n_ECG5_beat)>=VFB_min_bts
%             bpm_vec = ecg5_bpm1(1:end-1); % exclude the appended final beat
%         elseif length(n_abp_beats)>=VFB_min_bts
%             bpm_vec = abp_bpm(1:end-1); % exclude the appended final beat
%         elseif length(n_ppg_beats)>=VFB_min_bts
%             bpm_vec = ppg_bpm(1:end-1); % exclude the appended final beat
%         else
%             bpm_vec = nan;
%         end
%      
%         i_ecg_signal=[];
%         if ~isempty(ecg2_sig_sec) && max(ecg2_sig_sec)<inf
%             i_ecg_signal=ecg2_sig_sec;
%         elseif ~isempty(ecg5_sig_sec) && max(ecg5_sig_sec)<inf
%             i_ecg_signal=ecg5_sig_sec;
%         end
%             
%         % Trying to oscillatory waveform in 10-20 Hz Freq range
%         if ~isempty(i_ecg_signal)
%             [spec,sf] = specgram(i_ecg_signal,Fs);
%             power     = powercalc(spec,10,20,sf);
% 
%             alarmResult=1;
%             if power<VFB_pwr_th
%                 alarmResult=0;
%             end
%         else
%             alarmResult=logical(max(bpm_vec)>VFB_min_bpm_th-tolerance);
%         end
% 
% %%% END OF EXCERPT %%%

        
    otherwise
        error(['Unknown alarm type: ' alarm_type])
end

% Write result to answers.txt
fid = fopen(answers, 'a');
if (fid == -1)
    error('Could not open answer file');
end

% Get base name of record (without directories)
i = strfind(recordName, filesep);
if (~isempty(i))
    basename = recordName(i(end)+1 : end);
else
    basename = recordName;
end

fprintf(fid, '%s,%d\n', basename, alarmResult);
fclose(fid);

end

function vtach_result=vtach_detect(signal_sect, mxs, start_idx, Fs)
    mxs        = mxs-start_idx+1;
    % make sure peaks are located at local maxima
    pos        = chk_peaks(signal_sect, mxs, 5, 0.2*Fs);
    no_p_count = 0;
    % Define buffer of 100+/-20ms from each R peak
    qr_width   = round(0.12*0.5*Fs);
    rs_width   = qr_width;
    for i=1:length(pos)-1
        mx_a   = pos(i);
        mx_b   = pos(i+1);
        snip   = signal_sect(mx_a:mx_b);
        indent = mx_a;
        rr_int = mx_b-mx_a;
        bts    = Fs*60/rr_int;

        if rr_int>qr_width*3
            snip   = snip(rs_width:end-qr_width);
            indent = mx_a+rs_width;
        end
        sd_snip = std(snip);
        
        bnd_a   = (rr_int/Fs)*30+mx_a;
        bnd_b   = (rr_int/Fs)*135+mx_a;
        
        mx_bnd  = min([signal_sect(mx_a), signal_sect(mx_b)]);
        pk      = maxdet(snip,0.05*sd_snip);
        pk      = pk(pk>1);
        pk      = pk(snip(pk)<mx_bnd);
        peaks   = pk+indent;
        bnd_pks = peaks(bnd_a<peaks & peaks<bnd_b);
        
        found_nopwave = 0;
        if isempty(peaks)
            found_nopwave = 1;
        elseif length(peaks)==1
            found_nopwave = 0;
        elseif bts<50 % todo: should exclude this, implement back-search when R peak not found for X seconds
            found_nopwave = 5;
        elseif length(peaks)>1 && ~isempty(bnd_pks)
            found_nopwave = 0;
        end
        no_p_count = no_p_count+found_nopwave;
    end
    vtach_result=logical(no_p_count>=4); % once function included to main script, define variable = 4
end

function rpeaks=chk_peaks(win_signal, m, window, winsearch)
   rpeaks = [];
   maxvec = m(m>winsearch & m<length(win_signal)-winsearch);
   for mx=maxvec
       mx   = int64(mx);
       pre  = mean(win_signal(mx-window:mx));
       post = mean(win_signal(mx:mx+window));
       if pre<win_signal(mx) && post<win_signal(mx)
           rpeaks(end+1)=mx;
       else
           r_found=false;
           delta=0.25;
           if pre<post
               start=mx;
               first=true;
           else
               start=mx-winsearch+1;
               first=false;
           end
           search=win_signal(start:start+winsearch);
           while ~r_found && delta>0
               rvec=maxdet(search, delta);
               r_found=logical(length(rvec));
               if r_found
                   if first
                       rpeaks(end+1)=start+rvec(1);
                   else
                       rpeaks(end+1)=start+rvec(end);
                   end
               else
                   delta=delta-0.05;
               end
           end
       end
   end
   if ~isempty(rpeaks)
       rpeaks = sort(unique(rpeaks));
       dif    = [window+1,diff(rpeaks)];
       rpeaks = rpeaks(dif>2);
   end
end

function [spec,sf]=specgram(sig, Fs)
    sig     = resample(sig,5*Fs, length(sig));
    sum_sig = sum(sig.^2);
    scl_sig = sig/sqrt(sum_sig);
    spec    = fft(scl_sig);
    spec    = abs(spec(1:int64(length(spec)/2)));
    sf      = int64(length(spec)/(Fs/2));
end

function power=powercalc(specg, bound_a, bound_b, sf)
    power = sum(specg(bound_a*sf:bound_b*sf).^2);
end